#!/usr/bin/perl -w
# admin.pl -- Mako Hill <mako@debian.org>
# written for Organizers' Collaboaritve -- 2001
# this code is licensed unde the GNU GPL, see COPYING for more details

use strict;
use CGI qw/:standard/; # used for cookies
use Calf::Auth qw( :base );          # Site authentication module
use Calf::Config qw( :base $rec_thresh_editor );        # Site Config module
use Calf::Utils qw( :base :template :db );

## Open handlers to CGI and the database

my $dbh = Calf::Utils::open_db;
my $sth;
my $q = new CGI;
my $form_info = $q->Vars; # put the form info into a hash

# is the person logged in? $session_info is a refernce to a hash
# containing the username and session_id of the person logged in

my $session_info = &valid_id ($q, $dbh);
my $action = $form_info->{action};

$| = 1;

# chose which page to render in regards to the persons login status
# and the action they have requested..
# possible actions: login, logout, listview, itemview, default (see
# below)<
#
# then go and render that page

# if person is logged in
if (defined $session_info) {
  if ($form_info->{action} eq 'logout')
    { &logout; } 
  elsif ($form_info->{action} eq 'listview')
    { &listview ( $session_info ); }
  elsif ($form_info->{action} eq 'recent')
    { &recent(); } 
  else { &default( $session_info ); }
}
# if the user is not yet logged in, these are the only available options
elsif ($form_info->{action} eq 'login')
  { &login; }
else { &nologin }              # go to log in screen if not logged in

# disconnect from the database and end the program
$dbh->disconnect;


## SUBROUTINES
############################################
## Actions Defined for this Script
# possible actions are nologin, login, logout, listview,
# default

## nologin implies that the user is not currently logged in and has
## not requested a login with this request
sub nologin
  { # this will simply give them the login script
    &process_template('admin/login.html', {} ); }

## login is called when a person means to pass login information and
## log into the administerative interface
sub login {

  # grab input data from the CGI request
  my $username_unsafe = $form_info->{username};
  my $password_unsafe = $form_info->{password};
  my $item = $form_info->{item};
  my ( $db_password, $db_statement );

#  my $host = shift or die "usage: ok host\n";

  # pull the password out of the DB for the username entered
  $db_statement = qq/ SELECT password FROM
                      editors WHERE
                      username = "$username_unsafe" /;

  $db_password = ($dbh->selectrow_array( $db_statement ))[0]
    || &invalid_login;

  # check to see if the password matches the one in the DB
  if ( $username_unsafe ne ''        # if the username entered is not NULL
       && $password_unsafe ne ''     # or the password
       # and if the password/username entered is in the DB
       && $password_unsafe eq $db_password ) {

    # generate the session ID
    my $session_id = &generate_id;

    # add the session_id into the database entry for the user
    $db_statement = qq/ UPDATE editors
                        SET session_id = "$session_id",
                        last_active = NOW()
                        WHERE username = "$username_unsafe" /;
    $dbh->do( $db_statement )
      || &db_error( "do", $!, $db_statement );

    # no pull all the related other editor info out of the DB
    $db_statement = qq/ SELECT username, fullname,
			email, superuser
                        FROM editors
			WHERE session_id = "$session_id" /;
    $sth = $dbh->prepare( $db_statement )
      || &db_error( "prepare", $!, $db_statement );
    $sth->execute
      || &db_error( "execute", $!, $db_statement );
    my ( $db_username, $db_fullname, $db_email, $db_superuser )
      = ( $sth->fetchrow_array )[0,1,2,3];

    warn "$db_username, $db_fullname, $db_email, $db_superuser logged in.";
    # stick our new info into $session_info
    $session_info = { username => $db_username,
		      fullname => $db_fullname,
		      email => $db_email,
		      superuser => $db_superuser,
		      session_id => $session_id
		    };
    # pass session_info off to &default to generate the editors' homepage
    &default( $session_info ); # generate the default page
    return;                    # and finish up
  }

  # if any of the above conditions fail...
  else { &invalid_login }

  sub invalid_login {
    # generate the login page again
    my $vars = { error => 'invalid_login',
		 item => $item,
		 username => $username_unsafe };

    &process_template('admin/login.html', $vars );
    exit;
  }
}

## logout is called if you are logged in and you specifically call
## logout
sub logout {
  # create a launch a kill-cookie :)
  my $cookie = $q->cookie( -name => 'session_id',
			   -value => '',
			   -path => '/',
			   -expires => 'now' );
  print "Set-Cookie: $cookie\n";
  &nologin;  # call nologin and print the login screen
}

## listview is called for an editor to display every list within an
## editors categories. this is one of the first things i wrote whe i
## was starting this project and it's code is very immature. the nasty
## thing it returns at the end is the real icing on cake. then again,
## it does seem to do it's job
sub listview ($) {
  my ( $session_info ) = @_;
  my ( $type, %types );
  my $select_statement;
  my ( %codes, $code, @categories );
  my ( @lists_in_category );

  foreach $type ( @types ) {
    ( %codes, $code, @categories ) = ();   # zero out these vars for each type

    # grab the editor codes for each type in the DB, this should
    # probably done in one pass in before this sub
    my $tmp;
    %codes = %{ &retrieve_editor_codes( $session_info, $type, $dbh ) };

    # grab a list of lists for each code that the user is an editor for
    foreach $code ( keys( %codes ) ) {
      ( @lists_in_category ) = ();

      $select_statement = qq / SELECT /;

      my $i;
      for ( $i = 0; $i <= $#all_fields; $i++ ) {
	my $field = $all_fields[$i];
	$select_statement .= qq/ $field AS $field /;
	$select_statement .= "," unless $i == $#all_fields; 
      }
      $select_statement .= qq/ FROM calf_data
			       WHERE $type LIKE CONCAT("%","$code","%")
                               AND recommended <= $rec_thresh_editor /;

      $sth = $dbh->prepare( $select_statement )
	|| &db_error( "prepare", $!, $select_statement );
      $sth->execute
	|| &db_error( "execute", $!, $select_statement );

      while ( my $hash_ref = $sth->fetchrow_hashref( 'NAME_lc' ) ) {
	push @lists_in_category, { %$hash_ref };
      }

      # stick the lists, along with their code, the codes, desc, and
      # the type that we're searching by into a little package
      push @categories, { code => $code,
			  description => $codes{"$code"},
			  lists => [ @lists_in_category ],
			  type => $type };
    }
    # the abuse never stops. :) we stick THAT into another hash
    # creating a VERY long complex datastructure. if you're confused,
    # take a look at the example/diagram in the template
    $types{"$type"} = [ @categories ];
  }

  # throw this little monster at template and call it quits
  my $vars = { session_info => $session_info,
	       types => \%types };
  &process_cookie( $session_info->{session_id} );
  &process_template('admin/listview.html', $vars );
}


## default is called when you are logged in by login() and it is
## explicity called from many places in the site. it is referred to as
## the "editors' home" in many places in the templates
sub default ($) {
  my ( $session_info ) = @_;
  my ( @unapproved_lists, @rec_changed_lists );
  my ( @list_ids_un, @list_ids_rec ) = ();
  my ( %codes, $code, $type );
  my $select_statement;
  my %editor_for;

  # unforunately, this script mirrors the structure of
  # listview above in some ways (especially toward the top)
  foreach $type ( @types ) {
    %codes = %{ &retrieve_editor_codes( $session_info, $type, $dbh ) };
    $editor_for{"$type"} = { %codes };


    foreach $code ( keys %codes ) {
      # grab all unapproved lists out of the DBI that match the
      # editors codes which were retrieved directly above. at first we
      # only grab the id's themselve and not the entire lists
      $select_statement = qq/ SELECT
	                      id AS id
			      FROM calf_data
			      WHERE approved = 0
                              AND recommended <= $rec_thresh_editor
			    /;
      # filter by each code unless this a superuser and then just grab it all
      $select_statement .= qq/ AND $type LIKE CONCAT("%","$code","%") /
        unless ($session_info->{superuser} == 1);
      $sth = $dbh->prepare( $select_statement )
	|| &db_error( "prepare", $!, $select_statement );
      $sth->execute
	|| &db_error( "execute", $!, $select_statement );
      while ( my $hash_ref = $sth->fetchrow_hashref( 'NAME_lc' ) ) {
	# we check to see if we have already grabbed the list before
	# (some lists may show up twice if they are in two categories
	# that an editor edits for
	next if (grep /^$hash_ref->{id}$/, @list_ids_un);
	# push the list onto the list of id's we already dealt
	push @list_ids_un, $hash_ref->{id};
	# grab the rest of the info and stick it in the list of
	# unapproved lists
	push @unapproved_lists,
	  { %{ &grab_listinfo( $hash_ref->{id}, $dbh ) } };
      }

      # now we want to grab stuff modified in the last 48h that falls
      # within the editors domain (and provide a link to stuff farther
      # back (2 weeks?)
      $select_statement = qq/ SELECT
			      id AS id
			      FROM calf_data
                              WHERE
                              updated >= DATE_SUB( NOW(), INTERVAL 1 MONTH )
                              AND recommended <= $rec_thresh_editor
                            /;
      # filter by each code unless this a superuser and then just grab it all
      $select_statement .=  qq/ AND $type LIKE CONCAT("%","$code","%") /
	unless ($session_info->{superuser} == 1);
      $select_statement .=  qq{ ORDER BY updated };

      $sth = $dbh->prepare( $select_statement )
        || &db_error( "prepare", $!, $select_statement );
      $sth->execute
        || &db_error( "execute", $!, $select_statement );
      # see the comments on this section directly above as the method
      # it uses is mirrored above
      while ( my $hash_ref = $sth->fetchrow_hashref('NAME_lc') ) {
        next if (grep /^$hash_ref->{id}$/, @list_ids_rec);
        push @list_ids_rec, $hash_ref->{id};
        push @rec_changed_lists, 
          { %{ &grab_listinfo( $hash_ref->{id}, $dbh ) } };
      }

      last if ($session_info->{superuser} == 1); # exit early if it's
                                                 # a superuser as
                                                 # we'll grab
                                                 # everything in just
                                                 # one pass
    }
    last if ($session_info->{superuser} == 1);
  }

  # sort both of the lists of lists we've grabbed by relavant criteria
  @rec_changed_lists =
    sort { $b->{updated} cmp $a->{updated} } @rec_changed_lists;
  @unapproved_lists =
    sort { $a->{listname} cmp $b->{listname} } @unapproved_lists;

  # now we can hand it off to the template and go home
  my $vars = { session_info => $session_info,
	       unapproved_lists => \@unapproved_lists,
	       rec_changed_lists => \@rec_changed_lists,
	       editor_for => \%editor_for,
	       codes => \%codes,
	       descriptions => &build_descriptions( $dbh ),
	     };
  &process_cookie( $session_info->{session_id} );
  &process_template('admin/default.html', $vars );
}

## show a page of the recently change lists
sub recent {
  my ( @list_ids, @lists ) = ();
  my ( %codes, $code, $type );
  my $select_statement;

  # everything done here is sorted (and documented in the relavant
  # section that grabs recent lists from in sub default found directly
  # above

  # we'll grab all unapproved lists out of the DBI that match the
  # editors codes..
  foreach $type (@types) {
    %codes = %{ &retrieve_editor_codes( $session_info, $type, $dbh ) };

    foreach $code (keys %codes) {
      $select_statement = qq/ SELECT
        	              id AS id
			      FROM calf_data
			      WHERE recommended <= $rec_thresh_editor
              		      AND updated >= DATE_SUB(NOW(), INTERVAL 1 MONTH)
			    /;
      $select_statement .= qq/ AND $type LIKE CONCAT("%","$code","%") /
        unless ($session_info->{superuser} == 1);
      $select_statement .= qq/ ORDER BY updated /;

      $sth = $dbh->prepare( $select_statement )
        || &db_error( "prepare", $!, $select_statement );
      $sth->execute
        || &db_error( "execute", $!, $select_statement );
      while ( my $hash_ref = $sth->fetchrow_hashref('NAME_lc') ) {
	next if (grep /^$hash_ref->{id}$/, @list_ids);
        push @list_ids, $hash_ref->{id};
        push @lists,
          { %{ &grab_listinfo( $hash_ref->{id}, $dbh ) } };
      }
      last if ($session_info->{superuser} == 1);
    }
    last if ($session_info->{superuser} == 1);
  }

  @lists = sort { $b->{updated} cmp $a->{updated} } @lists;

  my $vars = { session_info => $session_info,
	       lists => \@lists,
	       descriptions => &build_descriptions( $dbh ),
	       sth => $select_statement
	     };

  &process_cookie( $session_info->{session_id} );
  &process_template('admin/recent.html', $vars );

}

## error -- a little wrapper around the dg_error script
sub error
  { &dg_error( 'script', 'admin', @_ ); }
